@@ -74,7 +74,7 @@ int passwordok(const char *s) {
 int main(int argc, char **argv){
   XEvent ev;
   KeySym ks;
-  char cbuf[10], rbuf[50];
+  char cbuf[10], rbuf[128]; /* shadow appears to suggest 127 a good value here */
   int clen, rlen=0;
   long goodwill= INITIALGOODWILL, timeout= 0;
   XSetWindowAttributes attrib;
@@ -99,9 +99,15 @@ int main(int argc, char **argv){
     pw->pw_passwd = sp->sp_pwdp;
   endspent();
 #endif
+
+  /* logically, if we need to do the following then the same 
+     applies to being installed setgid shadow.  
+     we do this first, because of a bug in linux. --jdamery */ 
+  setgid(getgid());
   /* we can be installed setuid root to support shadow passwords,
      and we don't need root privileges any longer.  --marekm */
   setuid(getuid());
+
   if (strlen(pw->pw_passwd) < 13) {
     fputs("password entry has no pwd\n",stderr); exit(1);
   }
@@ -191,7 +197,8 @@ int main(int argc, char **argv){
         break;
       default:
         if (clen != 1) break;
-        if (rlen < sizeof(rbuf)) rbuf[rlen]= cbuf[0];
+        if (rlen < (sizeof(rbuf) - 1)) rbuf[rlen]= cbuf[0];
+        /* allow space for the trailing \0 */
         rlen++;
         break;
       }
